@@ -40,10 +40,6 @@
  * GObject declarations
  */
 
-G_DEFINE_TYPE (ClipmanMenu, clipman_menu, GTK_TYPE_MENU)
-
-#define GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), CLIPMAN_TYPE_MENU, ClipmanMenuPrivate))
-
 struct _ClipmanMenuPrivate
 {
   GtkWidget            *mi_clear_history;
@@ -54,9 +50,12 @@ struct _ClipmanMenuPrivate
   gboolean              show_qr_code;
 #endif
   guint                 paste_on_activate;
+  guint                 max_menu_items;
   gboolean              never_confirm_history_clear;
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (ClipmanMenu, clipman_menu, GTK_TYPE_MENU)
+
 enum
 {
   REVERSE_ORDER = 1,
@@ -65,6 +64,7 @@ enum
 #endif
   PASTE_ON_ACTIVATE,
   NEVER_CONFIRM_HISTORY_CLEAR,
+  MAX_MENU_ITEMS,
 };
 
 static void             clipman_menu_finalize           (GObject *object);
@@ -133,6 +133,7 @@ cb_set_clipboard (GtkMenuItem *mi, const ClipmanHistoryItem *item)
   GtkClipboard *clipboard;
   ClipmanCollector *collector;
   ClipmanHistory *history;
+  gboolean add_primary_clipboard;
 
   switch (item->type)
     {
@@ -140,8 +141,15 @@ cb_set_clipboard (GtkMenuItem *mi, const ClipmanHistoryItem *item)
       clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
       gtk_clipboard_set_text (clipboard, item->content.text, -1);
 
-      clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
-      gtk_clipboard_set_text (clipboard, item->content.text, -1);
+      collector = clipman_collector_get ();
+      g_object_get (G_OBJECT (collector), "add-primary-clipboard", &add_primary_clipboard, NULL);
+      if (add_primary_clipboard)
+        {
+          g_warning ("sync primary clipboard");
+          clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
+          gtk_clipboard_set_text (clipboard, item->content.text, -1);
+        }
+      g_object_unref (collector);
       break;
 
     case CLIPMAN_HISTORY_TYPE_IMAGE:
@@ -164,60 +172,64 @@ cb_set_clipboard (GtkMenuItem *mi, const ClipmanHistoryItem *item)
       return;
     }
 
-  {
-    int dummyi;
-    KeySym key_sym;
-    KeyCode key_code;
+  cb_paste_on_activate (GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (mi), "paste-on-activate")));
+}
 
-    Display *display = XOpenDisplay (NULL);
-    if (display == NULL)
-      {
-        return;
-      }
-    else if (!XQueryExtension (display, "XTEST", &dummyi, &dummyi, &dummyi))
-      {
-        XCloseDisplay (display);
-        return;
-      }
+void
+cb_paste_on_activate (guint paste_on_activate)
+{
+  int dummyi;
+  KeySym key_sym;
+  KeyCode key_code;
 
-    switch (GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (mi), "paste-on-activate")))
-      {
-      case PASTE_INACTIVE:
-        break;
+  Display *display = XOpenDisplay (NULL);
+  if (display == NULL)
+    {
+      return;
+    }
+  else if (!XQueryExtension (display, "XTEST", &dummyi, &dummyi, &dummyi))
+    {
+      XCloseDisplay (display);
+      return;
+    }
 
-      case PASTE_CTRL_V:
-        key_sym = XK_Control_L;
-        key_code = XKeysymToKeycode (display, key_sym);
-        XTestFakeKeyEvent (display, key_code, True, CurrentTime);
-        key_sym = XK_v;
-        key_code = XKeysymToKeycode (display, key_sym);
-        XTestFakeKeyEvent (display, key_code, True, CurrentTime);
-        key_sym = XK_v;
-        key_code = XKeysymToKeycode (display, key_sym);
-        XTestFakeKeyEvent (display, key_code, False, CurrentTime);
-        key_sym = XK_Control_L;
-        key_code = XKeysymToKeycode (display, key_sym);
-        XTestFakeKeyEvent (display, key_code, False, CurrentTime);
-        break;
+  switch (paste_on_activate)
+    {
+    case PASTE_INACTIVE:
+      break;
 
-      case PASTE_SHIFT_INS:
-        key_sym = XK_Shift_L;
-        key_code = XKeysymToKeycode (display, key_sym);
-        XTestFakeKeyEvent (display, key_code, True, CurrentTime);
-        key_sym = XK_Insert;
-        key_code = XKeysymToKeycode (display, key_sym);
-        XTestFakeKeyEvent (display, key_code, True, CurrentTime);
-        key_sym = XK_Insert;
-        key_code = XKeysymToKeycode (display, key_sym);
-        XTestFakeKeyEvent (display, key_code, False, CurrentTime);
-        key_sym = XK_Shift_L;
-        key_code = XKeysymToKeycode (display, key_sym);
-        XTestFakeKeyEvent (display, key_code, False, CurrentTime);
-        break;
-      }
+    case PASTE_CTRL_V:
+      key_sym = XK_Control_L;
+      key_code = XKeysymToKeycode (display, key_sym);
+      XTestFakeKeyEvent (display, key_code, True, CurrentTime);
+      key_sym = XK_v;
+      key_code = XKeysymToKeycode (display, key_sym);
+      XTestFakeKeyEvent (display, key_code, True, CurrentTime);
+      key_sym = XK_v;
+      key_code = XKeysymToKeycode (display, key_sym);
+      XTestFakeKeyEvent (display, key_code, False, CurrentTime);
+      key_sym = XK_Control_L;
+      key_code = XKeysymToKeycode (display, key_sym);
+      XTestFakeKeyEvent (display, key_code, False, CurrentTime);
+      break;
 
-    XCloseDisplay (display);
-  }
+    case PASTE_SHIFT_INS:
+      key_sym = XK_Shift_L;
+      key_code = XKeysymToKeycode (display, key_sym);
+      XTestFakeKeyEvent (display, key_code, True, CurrentTime);
+      key_sym = XK_Insert;
+      key_code = XKeysymToKeycode (display, key_sym);
+      XTestFakeKeyEvent (display, key_code, True, CurrentTime);
+      key_sym = XK_Insert;
+      key_code = XKeysymToKeycode (display, key_sym);
+      XTestFakeKeyEvent (display, key_code, False, CurrentTime);
+      key_sym = XK_Shift_L;
+      key_code = XKeysymToKeycode (display, key_sym);
+      XTestFakeKeyEvent (display, key_code, False, CurrentTime);
+      break;
+    }
+
+  XCloseDisplay (display);
 }
 
 static void
@@ -266,6 +278,30 @@ cb_clear_history (ClipmanMenu *menu)
   gtk_clipboard_clear (clipboard);
 }
 
+static void
+cb_launch_clipman_bin (ClipmanMenu *menu,
+                       gpointer     user_data)
+{
+  GError *error = NULL;
+  GtkWidget *error_dialog;
+  GtkWidget *mi = GTK_WIDGET (user_data);
+
+  if (g_strcmp0 (gtk_menu_item_get_label (GTK_MENU_ITEM (mi)), "_Show full history...") == 0)
+    g_spawn_command_line_async ("xfce4-clipman-history", &error);
+  else
+    g_spawn_command_line_async ("xfce4-clipman-settings", &error);
+
+  if (error != NULL)
+  {
+    error_dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                                           _("Unable to open the clipman history dialog"));
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog), "%s", error->message);
+    gtk_dialog_run (GTK_DIALOG (error_dialog));
+    gtk_widget_destroy (error_dialog);
+    g_error_free (error);
+  }
+}
+
 /*
  * Private methods
  */
@@ -291,6 +327,7 @@ _clipman_menu_update_list (ClipmanMenu *menu)
   const ClipmanHistoryItem *item_to_restore;
   GSList *list, *l;
   gint pos = 0;
+  gint i = 0;
 
   /* Get the most recent item in the history */
   item_to_restore = clipman_history_get_item_to_restore (menu->priv->history);
@@ -305,8 +342,10 @@ _clipman_menu_update_list (ClipmanMenu *menu)
   list = clipman_history_get_list (menu->priv->history);
   if (menu->priv->reverse_order)
     list = g_slist_reverse (list);
-  for (l = list; l != NULL; l = l->next)
+  for (i = 0, l = list; i < menu->priv->max_menu_items; i++, l = l->next)
     {
+      if (l == NULL)
+        break;
       item = l->data;
 
       switch (item->type)
@@ -425,8 +464,6 @@ clipman_menu_class_init (ClipmanMenuClass *klass)
 {
   GObjectClass *object_class;
 
-  g_type_class_add_private (klass, sizeof (ClipmanMenuPrivate));
-
   clipman_menu_parent_class = g_type_class_peek_parent (klass);
 
   object_class = G_OBJECT_CLASS (klass);
@@ -463,6 +500,13 @@ clipman_menu_class_init (ClipmanMenuClass *klass)
                                                          "Set to FALSE to clear the history list with confirmation",
                                                          FALSE,
                                                          G_PARAM_CONSTRUCT|G_PARAM_READWRITE));
+
+  g_object_class_install_property (object_class, MAX_MENU_ITEMS,
+                                   g_param_spec_uint ("max-menu-items",
+                                                      "MaxMenuItems",
+                                                      "Maximum amount of items displayed in the plugin's menu",
+                                                      1, 100, 15,
+                                                      G_PARAM_CONSTRUCT|G_PARAM_READWRITE));
 }
 
 static void
@@ -470,8 +514,9 @@ clipman_menu_init (ClipmanMenu *menu)
 {
   GtkWidget *mi;
   GtkWidget *image;
+  guint max_texts_in_history;
 
-  menu->priv = GET_PRIVATE (menu);
+  menu->priv = clipman_menu_get_instance_private (menu);
 
   /* ClipmanHistory */
   menu->priv->history = clipman_history_get ();
@@ -483,6 +528,14 @@ clipman_menu_init (ClipmanMenu *menu)
   mi = gtk_separator_menu_item_new ();
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 
+  max_texts_in_history = clipman_history_get_max_texts_in_history (menu->priv->history);
+  if (max_texts_in_history > menu->priv->max_menu_items)
+    {
+      mi = gtk_menu_item_new_with_mnemonic (_("_Show full history..."));
+      gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+      g_signal_connect_swapped (mi, "activate", G_CALLBACK (cb_launch_clipman_bin), mi);
+    }
+
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   menu->priv->mi_clear_history = mi = gtk_image_menu_item_new_with_mnemonic (_("_Clear history"));
   image = gtk_image_new_from_icon_name ("edit-clear-symbolic", GTK_ICON_SIZE_MENU);
@@ -491,6 +544,10 @@ G_GNUC_END_IGNORE_DEPRECATIONS
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   g_signal_connect_swapped (mi, "activate", G_CALLBACK (cb_clear_history), menu);
 
+  mi = gtk_menu_item_new_with_mnemonic (_("_Clipman settings..."));
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+  g_signal_connect_swapped (mi, "activate", G_CALLBACK (cb_launch_clipman_bin), mi);
+
   /* Show all the items */
   gtk_widget_show_all (GTK_WIDGET (menu));
 }
@@ -530,6 +587,10 @@ clipman_menu_set_property (GObject *object,
       priv->never_confirm_history_clear = g_value_get_boolean (value);
       break;
 
+    case MAX_MENU_ITEMS:
+      priv->max_menu_items = g_value_get_uint (value);
+      break;
+
     default:
       break;
     }
@@ -557,6 +618,10 @@ clipman_menu_get_property (GObject *object,
       g_value_set_boolean (value, priv->never_confirm_history_clear);
       break;
 
+    case MAX_MENU_ITEMS:
+      g_value_set_uint (value, priv->max_menu_items);
+      break;
+
     default:
       break;
     }
