@@ -19,10 +19,6 @@
 #ifndef __CLIPMAN_PLUGIN_H__
 #define __CLIPMAN_PLUGIN_H__
 
-#if ! defined (PANEL_PLUGIN) && ! defined (STATUS_ICON)
-#error "None of PANEL_PLUGIN or STATUS_ICON is defined for preprocessing"
-#endif
-
 #include <gtk/gtk.h>
 #include <xfconf/xfconf.h>
 
@@ -32,6 +28,7 @@
 
 #include <x11-clipboard-manager/daemon.h>
 #include "actions.h"
+#include "common.h"
 #include "collector.h"
 #include "history.h"
 
@@ -59,6 +56,10 @@ struct _MyPlugin
   GtkWidget            *popup_menu;
   gulong                popup_menu_id;
   GtkApplication       *app;
+  GtkWidget            *dialog;
+  GtkWidget            *treeview;
+  GtkWidget            *submit_button;
+  GtkWidget            *entry;
 };
 
 /*
